<?php

namespace WHMCS\Module\Server\VirtFusionDirect;

use WHMCS\Database\Capsule as DB;
use WHMCS\Module\Server\VirtFusionDirect\Curl;

class ModuleFunctions extends Module
{
    public function __construct()
    {
        parent::__construct();
    }

    /**
     *
     * CREATE SERVER
     *
     * Before creating a server, we check to see if a user exists in VirtFusion that matches
     * the WHMCS user. If it matches, We move on to create the server, if not, we attempt to
     * create a user to assign to the new server.
     *
     */
    public function createAccount($params)
    {
        try {

            /**
             *
             * If the service exists in the custom table, Cancel the create account action.
             *
             */
            if ($this->serviceExistsInCustomTable($params['serviceid'])) {
                return 'Service already exists. You must run a termination first.';
            }


            /**
             *
             * If no VirtFusionDirect control server exists, cancel the create account action.
             *
             */
            $cp = $this->getCP(false);

            if (!$cp) {
                return 'No Control server found.';
            }

            $this->log(__FUNCTION__ . ' (source params)', $params, '');

            /**
             *
             * Does a user account in VirtFusion match this account (byExtRelationId) in WHMCS.
             *
             */
            $request = $this->initCurl($cp['token']);
            $data = $request->get($cp['url'] . '/users/' . $params['userid'] . '/byExtRelation');
            $this->log(__FUNCTION__ . ' (check user)', $request->getRequestInfo(), $data);

            switch ($request->getRequestInfo('http_code')) {
                case 200:

                    /**
                     *
                     * A user with relation ID exists in VirtFusion. We can provision under that account.
                     *
                     */
                    break;

                case 404:

                    /**
                     *
                     * A user doesn't exist in VirtFusion. We should attempt to create one.
                     *
                     */
                    $user = DB::table('tblclients')->where('id', $params['userid'])->first();

                    $request = $this->initCurl($cp['token']);

                    $request->addOption(CURLOPT_POSTFIELDS, json_encode(
                        [
                            "name" => $user->firstname . ' ' . $user->lastname,
                            "email" => $user->email,
                            "extRelationId" => $user->id
                        ]
                    ));

                    $data = $request->post($cp['url'] . '/users');

                    $this->log(__FUNCTION__ . ' (create user request)', $request->getRequestInfo(), $data);

                    if ($request->getRequestInfo('http_code') !== 201) {
                        return 'Unable to create user.';
                    }
                    break;
                default:
                    return 'Error processing user account.';
                    break;
            }

            $data = json_decode($data);

            /**
             *
             * A user is available. We can now attempt to create a server.
             *
             */
            $request = $this->initCurl($cp['token']);
            $request->addOption(CURLOPT_POSTFIELDS, json_encode(
                [
                    "packageId" => $params['configoption2'],
                    "userId" => $data->data->id,
                    "hypervisorId" => $params['configoption1'],
                    "ipv4" => $params['configoption3'],
                ]
            ));

            $data = $request->post($cp['url'] . '/servers');

            $data = json_decode($data);

            $this->log(__FUNCTION__ . '(create server)', $request->getRequestInfo(), $data);

            if ($request->getRequestInfo('http_code') === 201) {
                DB::table($this->dbTable)->insert(['service_id' => $params['serviceid'], 'server_id' => $data->data->id]);

                /**
                 *
                 * Server was created successfully.
                 *
                 */
                return 'success';
            } else {
                if ($data->errors[0]) {
                    return $data->errors[0];
                }
                return 'Unknown error.';
            }
        } catch (\Exception $e) {
            $this->log(__FUNCTION__, $params, $e->getMessage(), $e->getTraceAsString());
            return $e->getMessage();
        }
    }

    /**
     *
     * TERMINATE SERVER
     *
     * When requesting to terminate a server in VirtFusion, we leave it set to
     * the default 5-minute delay allowing to un-terminate in VirtFusion if the
     * request was done in error.
     *
     */
    public function terminateAccount($params)
    {
        $service = DB::table($this->dbTable)->where('service_id', $params['serviceid'])->first();

        if ($service) {

            $whmcsService = DB::table('tblhosting')->where('id', $params['serviceid'])->first();

            $cp = $this->getCP($whmcsService->server);
            $request = $this->initCurl($cp['token']);
            $data = $request->delete($cp['url'] . '/servers/' . $service->server_id);
            $data = json_decode($data);
            $this->log(__FUNCTION__, $request->getRequestInfo(), $data);

            switch ($request->getRequestInfo('http_code')) {

                case 204:
                    DB::table($this->dbTable)->where('service_id', $params['serviceid'])->delete();
                    return 'success';
                    break;

                case 404:
                    if (property_exists($data, 'msg')) {
                        if ($data->msg == 'server not found') {
                            $this->deleteServiceFromCustomTable($params['serviceid']);
                            return 'success';
                        } else {
                            return '404 was returned from the web service with the msg property but doesn\'t contain appropriate data to process a termination.';
                        }
                    } else {
                        return '404 was returned from the web service without the msg property. The service may be currently unavailable.';
                    }
                    break;

                default:
                    return 'Termination request failed. The web service reported HTTP code ' . $request->getRequestInfo('http_code');
                    break;
            }
        }
        return 'Service not found. Termination routine has already been run?';
    }

    /**
     *
     * SUSPEND SERVER
     *
     * When requesting to suspend a server in VirtFusion it may be delayed if another action
     * is being processed. This function will return success if the server is either suspended
     * now or has been queued for suspension.
     *
     */
    public function suspendAccount($params)
    {
        $service = DB::table($this->dbTable)->where('service_id', $params['serviceid'])->first();

        if ($service) {
            $whmcsService = DB::table('tblhosting')->where('id', $params['serviceid'])->first();

            $cp = $this->getCP($whmcsService->server);
            $request = $this->initCurl($cp['token']);
            $data = $request->post($cp['url'] . '/servers/' . $service->server_id . '/suspend');
            $data = json_decode($data);
            $this->log(__FUNCTION__, $request->getRequestInfo(), $data);

            switch ($request->getRequestInfo('http_code')) {

                case 204:
                    return 'success';
                    break;

                case 404:
                    if (property_exists($data, 'msg')) {
                        if ($data->msg == 'server not found') {
                            $this->deleteServiceFromCustomTable($params['serviceid']);
                            return 'success';
                        } else {

                            return '404 was returned from the web service with the msg property but doesn\'t contain appropriate data to process a suspension.';
                        }
                    } else {
                        return '404 was returned from the web service without the msg property. The service may be currently unavailable.';
                    }
                    break;
                case 423:
                    if (property_exists($data, 'msg')) {
                        return $data->msg;
                    }

                default:
                    return 'Suspend request failed. The web service reported HTTP code ' . $request->getRequestInfo('http_code');
                    break;
            }
        }
        return 'Service not found.';
    }

    public function unsuspendAccount($params)
    {
        $service = DB::table($this->dbTable)->where('service_id', $params['serviceid'])->first();

        if ($service) {
            $whmcsService = DB::table('tblhosting')->where('id', $params['serviceid'])->first();

            $cp = $this->getCP($whmcsService->server);
            $request = $this->initCurl($cp['token']);
            $data = $request->post($cp['url'] . '/servers/' . $service->server_id . '/unsuspend');
            $data = json_decode($data);
            $this->log(__FUNCTION__, $request->getRequestInfo(), $data);

            switch ($request->getRequestInfo('http_code')) {

                case 204:
                    return 'success';
                    break;

                case 404:
                    if (property_exists($data, 'msg')) {
                        if ($data->msg == 'server not found') {
                            $this->deleteServiceFromCustomTable($params['serviceid']);
                            return 'success';
                        } else {
                            return '404 was returned from the web service with the msg property but doesn\'t contain appropriate data to process an unsuspension.';
                        }
                    } else {
                        return '404 was returned from the web service without the msg property. The service may be currently unavailable.';
                    }
                    break;
                case 423:
                    if (property_exists($data, 'msg')) {
                        return $data->msg;
                    }

                default:
                    return 'Unsuspend request failed. The web service reported HTTP code ' . $request->getRequestInfo('http_code');
                    break;
            }
        }
        return 'Service not found';
    }

    public function adminServicesTabFields($params)
    {
        $serverId = '';

        $service = DB::table($this->dbTable)->where('service_id', $params['serviceid'])->first();

        if ($service) {
            $serverId = $service->server_id;
        }
        $fields = [
            'Server ID' => '<input type="text" name="modulefields[0]" size="20" value="' . $serverId . '" /><span class="text-info">&nbsp;&nbsp;Changing the Sever ID manually is not recommended. Alterations to this field are usually handled automatically.</span>',
        ];

        if ($params['status'] != 'Terminated') {
            $fields['Options'] = '<button onclick="loginAsServerOwner()" type="button" class="btn btn-primary">Login as Server Owner</button><script>function loginAsServerOwner() {$.ajax({type: "GET",dataType: "json",url: "/modules/servers/VirtFusionDirect/admin.php?serviceID=' . $params['serviceid'] . '&action=loginAsServerOwner"}).done(function (data) {if (data.success) {if (data.token_url) {window.open(data.token_url);}}}).fail(function (data) {}).always(function (data) {});}</script>';
        }

        return $fields;
    }

    public function adminServicesTabFieldsSave($params)
    {

        if ($_POST['modulefields'][0] == '') {
            $this->deleteServiceFromCustomTable($params['serviceid']);
        } else {
            DB::table($this->dbTable)->updateOrInsert(
                [
                    "service_id" => $params['serviceid']
                ],
                [
                    'server_id' => $_POST['modulefields'][0]
                ]
            );
        }
    }
}